/**
 *  Copyright (c) Members of the EGEE Collaboration. 2004. 
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.  
 *
 *  Licensed under the Apache License, Version 2.0 (the "License"); 
 *  you may not use this file except in compliance with the License. 
 *  You may obtain a copy of the License at 
 *
 *      http://www.apache.org/licenses/LICENSE-2.0 
 *
 *  Unless required by applicable law or agreed to in writing, software 
 *  distributed under the License is distributed on an "AS IS" BASIS, 
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 *  See the License for the specific language governing permissions and 
 *  limitations under the License.
 *
 *  GLite Data SRM Util CPP: Context Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>, Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: Context.h,v 1.5 2008/11/20 11:44:47 ptedesco Exp $
 *  Release: $Name: glite-data-srm-util-cpp_R_2_0_0_1 $
 *
 */
#ifndef GLITE_DATA_SRM_UTIL_CPP_CONTEXT_H_
#define GLITE_DATA_SRM_UTIL_CPP_CONTEXT_H_

#include <string>
#include <vector>
#include <boost/shared_ptr.hpp>

namespace glite {
namespace data  {
namespace srm   {
namespace util  {

/**
 * Information about an SRM call to be used for logging
 */
class CallInfo{
public:    
    /**
     * key-value pair
     */
    struct AdditionalInfo{
        AdditionalInfo(const std::string& k,const std::string& v) :
            key(k),
            value(v){
        }
        std::string key;
        std::string value;
    };
    // list of key-value pairs, containing e.g. info about SURLs
    typedef std::vector<AdditionalInfo> AdditionalInfos;
    
    /**
     * default constructor
     */
    CallInfo(){
    }
    /**
     * constructor
     * @param token [IN] the request token
     */
    explicit CallInfo(const std::string& token) : m_token(token){
    }
    
    /**
     * set the request token
     * @param value [IN] request token
     * @return CallInfo object 
     */
    CallInfo& token(const std::string& value){
        m_token = value;
        return (*this);
    }
    
    /**
     * add a key value pair
     * @param key [IN]
     * @param value [IN]
     * @return CallInfo object 
     */
    CallInfo& add(const std::string& key,const std::string& value){
        m_additionalInfos.push_back(AdditionalInfo(key,value));
        return (*this);
    }
    
    /**
     * get the token
     * @return token
     */
    const std::string& token() const {
        return m_token;
    }
    
    /**
     * get the list of key-value pairs
     * @return list of key-value pairs
     */
    const AdditionalInfos& additionalInfos() const{
        return m_additionalInfos;
    }
    
protected:
    // token
    std::string m_token;
    // key-value pairs
    AdditionalInfos m_additionalInfos;
};    
    
/**
 * SRM Context Class
 */
struct Context{
    
    /**
     * The Context Pointer type
     */
    typedef boost::shared_ptr<Context> Ptr;
    
    /**
     * SRM endpoint to be used
     */
    std::string endpoint;
    /**
     * SRM interface version
     */
    std::string version;
    /**
     * Timeout for HTTP calls
     */
    unsigned long httpTimeout;
    
    /**
     * If some features are missing in the required SRM interface version,
     * ignore them, otherwise throw an exception
     */
    bool ignoreMissingFeatures;
    
    /**
     * beforeCall
     * @param method [IN] method name
     */
    void beforeCall(
        const char * method) throw();
    
    /**
     * onSuccess
     * @param method [IN] method name
     * @param address [IN] ip address
     * @param info [IN] SRM call information
     */
    void onSuccess(
        const char * method,
        const char * address,
        const CallInfo * info) throw();
    
    /**
     * onFailure
     * @param method [IN] method name
     * @param address [IN] ip address
     * @param info [IN] SRM call information
     */
    void onFailure(
        const char * method,
        const char * address,
        const CallInfo * info) throw();
    
    /**
     * onWarning
     * @param scope [IN] 
     * @param description [IN]
     * @param address [IN] ip address
     * @param info [IN] SRM call information
     */
    void onWarning(
        const char * scope,
        const char * description,
        const char * address) throw();

protected:

    /**
     * Constructor
     * declared protected just to avoid direct instantiation
     * and ensure that clients derive from this class (even 
     * without overriding anything)
     */    
    Context();
    
    /**
     * a virtual destructor just to silence gcc's warning
     * (mostly harmless anyway)
     */
    virtual ~Context(){
    }
        
    /**
     * before_call
     * to be overridden by derived classes
     * @param method [IN] method name
     */
    virtual void before_call(
        const char * method){
    }
    
    /**
     * on_success
     * to be overridden by derived classes
     * @param method [IN] method name
     * @param address [IN] ip address
     * @param info [IN] SRM call information
     */
    virtual void on_success(
        const char * method,
        const char * address,
        const CallInfo * info){
    }
    
    /**
     * on_failure
     * to be overridden by derived classes
     * @param method [IN] method name
     * @param address [IN] ip address
     * @param info [IN] SRM call information
     */
    virtual void on_failure(
        const char * method,
        const char * address,
        const CallInfo * info){
    }
    
    /**
     * on_warning
     * to be overridden by derived classes
     * @param scope [IN] 
     * @param description [IN]
     * @param address [IN] ip address
     * @param info [IN] SRM call information
     */
    virtual void on_warning(
        const char * scope,
        const char * description,
        const char * address){
    }
            
};


} // End namespace util
} // End namespace srm
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_SRM_UTIL_CPP_CONTEXT_H_
