/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004. 
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.  
 *
 *  Licensed under the Apache License, Version 2.0 (the "License"); 
 *  you may not use this file except in compliance with the License. 
 *  You may obtain a copy of the License at 
 *
 *      http://www.apache.org/licenses/LICENSE-2.0 
 *
 *  Unless required by applicable law or agreed to in writing, software 
 *  distributed under the License is distributed on an "AS IS" BASIS, 
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 *  See the License for the specific language governing permissions and 
 *  limitations under the License.
 *
 *  GLite Data SRM Util CPP: Request Class
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: AsynchRequest.h,v 1.3 2008/03/13 16:32:44 ptedesco Exp $
 *  Release: $Name: glite-data-srm-util-cpp_R_2_0_0_1 $
 *
 */
#ifndef GLITE_DATA_SRM_UTIL_CPP_DETAILS_ASYNCHREQUEST_H_
#define GLITE_DATA_SRM_UTIL_CPP_DETAILS_ASYNCHREQUEST_H_

#include "glite/data/srm/util/details/Request.h"

namespace glite  {
namespace data   {
namespace srm    {
namespace util   {
namespace details{

/**
 * Abstract Class for SRM Asynchronous requests
 */ 
class AsynchRequest: public Request {
public: 

    /**
     * Return the Asynchronous request token
     */
    const std::string& token() const {return m_token;}

    // The AsynchRequest properies
    // -------------------------------------------------------------------------
    SrmStatus  status;
    
    // -------------------------------------------------------------------------
    // The AsynchRequest class abstract methods
    // -------------------------------------------------------------------------
    
    /**
     * Execute the Request. The result will be stored in the Request's status
     * field. Exception are raised in case of errors. Please note that the 
     * request's failure is not considered an error, therefore exception are 
     * not raised, instead the request status and result properties will be 
     * updated accordingly
     * @throw LogicError in case a logic error (e.g. precondityion not met) happens
     * @throw RemoteError in case it cannot contact the remote SRM
     * @throw SecurityError in case a security error happens in contacting the SRM
     * @throw SrmBadResponse in case the SRM returned an inconsistent response
     * @throw SrmError in case it cannot contact the remote SRM
     */
    virtual void execute() /* throw (LogicError, RemoteError, SecurityError, SrmBadResponse, SrmError)*/ = 0;
    
    /**
     * Update the status of the Request.
     * Exception are raised in case of errors. Please note that the 
     * request's failure is not considered an error, therefore exception are 
     * not raised, instead the request status and result properties will be 
     * updated accordingly
     * @throw LogicError in case a logic error (e.g. precondityion not met) happens
     * @throw RemoteError in case it cannot contact the remote SRM
     * @throw SecurityError in case a security error happens in contacting the SRM
     * @throw SrmBadResponse in case the SRM returned an inconsistent response
     * @throw SrmError in case it cannot contact the remote SRM
     */
    virtual void update() /*throw (LogicError, RemoteError, SecurityError, SrmBadResponse, SrmError)*/ = 0;

    /**
     * Complete a Request and release the allocated resources.
     * Exception are raised in case of errors. Please note that the 
     * request's failure is not considered an error, therefore exception are 
     * not raised, instead the request status and result properties will be 
     * updated accordingly
     * @throw LogicError in case a logic error (e.g. precondityion not met) happens
     * @throw RemoteError in case it cannot contact the remote SRM
     * @throw SecurityError in case a security error happens in contacting the SRM
     * @throw SrmBadResponse in case the SRM returned an inconsistent response
     * @throw SrmError in case it cannot contact the remote SRM
     */
    virtual void done() /*throw (LogicError, RemoteError, SecurityError, SrmBadResponse, SrmError)*/ = 0;

    /**
     * Abort a Request and cleanup the allocated resources.
     * Exception are raised in case of errors. Please note that the 
     * request's failure is not considered an error, therefore exception are 
     * not raised, instead the request status and result properties will be 
     * updated accordingly
     * @throw LogicError in case a logic error (e.g. precondityion not met) happens
     * @throw RemoteError in case it cannot contact the remote SRM
     * @throw SecurityError in case a security error happens in contacting the SRM
     * @throw SrmBadResponse in case the SRM returned an inconsistent response
     * @throw SrmError in case it cannot contact the remote SRM
     */
    virtual void abort() /*throw (LogicError, RemoteError, SecurityError, SrmBadResponse, SrmError)*/ = 0;
    
protected:
    
    /**
     * destructor
     */
    virtual ~AsynchRequest(){}
        
    /**
     * Constructor
     */
    explicit AsynchRequest(Context::Ptr ctx):Request(ctx){}

    /**
     * Constructor
     */
    AsynchRequest(Context::Ptr ctx, const std::string& token):Request(ctx),m_token(token){}

    /**
     * Check the token status
     * @throw LogicError is the condition is not satisfied
     */
    void checkToken(bool should_be_assigned) /*throw(LogicError)*/{
        if(m_token.empty() == should_be_assigned){
            const char * reason = 
                (true == should_be_assigned)?"the request token is empty"
                :"the request token is already assigned";
            throw LogicError(reason);
        }
    }
        
private:
    /**
     * Default Constructor. Not Implemented
     */
    AsynchRequest();

protected:
    /**
     * The Request Token
     */
    std::string m_token;
};

} // End namespace details
} // End namespace util
} // End namespace srm
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_SRM_UTIL_CPP_DETAILS_ASYNCHREQUEST_H_
