/*
 *  Copyright (c) Members of the EGEE Collaboration. 2004. 
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.  
 *
 *  Licensed under the Apache License, Version 2.0 (the "License"); 
 *  you may not use this file except in compliance with the License. 
 *  You may obtain a copy of the License at 
 *
 *      http://www.apache.org/licenses/LICENSE-2.0 
 *
 *  Unless required by applicable law or agreed to in writing, software 
 *  distributed under the License is distributed on an "AS IS" BASIS, 
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 *  See the License for the specific language governing permissions and 
 *  limitations under the License.
 *
 *  GLite Data SRM Util CPP: RequestFactory
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>
 *  Version info: $Id: RequestFactory.h,v 1.8 2007/11/05 08:55:50 ptedesco Exp $
 *  Release: $Name: glite-data-srm-util-cpp_R_2_0_0_1 $
 *
 */
#ifndef GLITE_DATA_SRM_UTIL_CPP_DETAILS_REQUEST_FACTORY_H_
#define GLITE_DATA_SRM_UTIL_CPP_DETAILS_REQUEST_FACTORY_H_

#include <string>
#include <map>
#include <boost/utility.hpp>

namespace glite  {
namespace data   {
namespace srm    {
namespace util   {
namespace details{
    
/**
 * Store instances for a given tag
 */
template <typename T>
class InstanceStorage : boost::noncopyable{
public:
    typedef std::map<std::string, T *> Map;
    
    /**
     * Register an Instace
     */
    void registerInstance(const std::string& tag, T * t) /*throw(LogicError)*/ {
        std::pair<typename Map::iterator, bool> result = 
            m_instances.insert(typename Map::value_type(strip_tag(tag),t));
        if(false == result.second){
            throw LogicError("a instance for this tag already exist");
        }
    }

    /**
     * Deregister an Instance
     */
    void deregisterInstance(const std::string& tag, T * t){
        typename Map::iterator it = m_instances.find(strip_tag(tag));
        if(it != m_instances.end()){
            if((*it).second == t){
                m_instances.erase(it);
            }
        }
    }
    
    /**
     * Find an instance
     */
    T * find(const std::string& tag){
        T * t = 0;
        typename Map::iterator it = m_instances.find(strip_tag(tag));
        if(it != m_instances.end()){
            t = (*it).second;
        }
        return t;
    }
private:
    
    /**
     * remove the path number from the tag
     */
    std::string strip_tag(const std::string& tag) throw (){
        std::string stripped = tag;
        int n = std::count(tag.begin(),tag.end(),'.');
        if(n > 1){
            n = stripped.find_last_of('.');
            stripped = stripped.substr(0,n);
        }
        return stripped;
    }
private:
    /**
     * The stored instances
     */
    Map  m_instances;
};

/**
 * Base class for Request Factories
 */
template <typename T>
class RequestFactory : boost::noncopyable{
public:

    /**
     * Factory Method: create a new instance of a Request given the
     * version specified on the Context
     */
    static T * create(Context::Ptr ctx) /*throw (LogicError)*/{
        RequestFactory * f = s_instances.find(ctx->version);
        if(0 == f){
            // Create reason
            std::string reason = "Invalid SRM version [" + ctx->version + "] for endpoint [" + ctx->endpoint + "]"; 
            throw LogicError(reason);
        }
        return f->createRequest(ctx);
    }

    /**
     * Static Method: Check if a Request type in supported in the
     * version specified on the Context
     */
    static bool isSupported(Context::Ptr ctx) /*throw ()*/{
        bool result = false;
        RequestFactory * f = s_instances.find(ctx->version);
        if(0 != f){
            result = f->isRequestSupported(ctx);
        }
        return result;
    }
    
protected:

    /**
     * Constructor
     */
    explicit RequestFactory(const std::string& v):m_version(v){
        try{
            s_instances.registerInstance(m_version,this);
        }catch(const LogicError&){
            // Ignore registration errors 
        }
    }

    /**
     * Destructor
     */
    virtual ~RequestFactory(){
        s_instances.deregisterInstance(m_version,this);
    }

    /**
     * Virtual Method: create a new instance of a Request given the
     * version specified on the Context
     */
    virtual T * createRequest(Context::Ptr ctx) /*throw (LogicError)*/ = 0;

    /**
     * Virtual Method: Check if a Request type in supported in the
     * version specified on the Context
     */
    virtual bool isRequestSupported(Context::Ptr ctx) /*throw ()*/ = 0;

private:

    /**
     * Default Constructor. Not Implemented
     */
    RequestFactory();

private:

    static InstanceStorage<RequestFactory> s_instances;

    /**
     * Version
     */
    std::string m_version;
};

/**
 * Base class for Asynch Request Factories
 */
template <typename T>
class AsynchRequestFactory : boost::noncopyable{
public:

    /**
     * Factory Method: create a new instance of a Request given the
     * version specified on the Context
     */
    static T * create(Context::Ptr ctx, const std::string& token) /*throw (LogicError)*/{
        AsynchRequestFactory * f = s_instances.find(ctx->version);
        if(0 == f){
            // Create reason
            std::string reason = "Invalid SRM version [" + ctx->version + "] for endpoint [" + ctx->endpoint + "]"; 
            throw LogicError(reason);
        }
        return f->createRequest(ctx,token);
    }

protected:

    /**
     * Constructor
     */
    explicit AsynchRequestFactory(const std::string& v):m_version(v){
        try{
            s_instances.registerInstance(m_version,this);
        }catch(const LogicError&){
            // Ignore registration errors 
        }
    }

    /**
     * Destructor
     */
    virtual ~AsynchRequestFactory(){
        s_instances.deregisterInstance(m_version,this);
    }

    /**
     * Virtual Method: create a new instance of a Request given the
     * version specified on the Context
     */
    virtual T * createRequest(Context::Ptr ctx, const std::string& token) /*throw (LogicError)*/ = 0;

private:

    /**
     * Default Constructor. Not Implemented
     */
    AsynchRequestFactory();

private:

    static InstanceStorage<AsynchRequestFactory> s_instances;
    
    /**
     * Version
     */
    std::string m_version;
};

// Some Helper Methods

/*
 * Define the Factory Storage
 */
/* 
 * The way static member inside a template is instanciated has
 * changed between gcc v 3.2.3 and 3.4, so one has to do a 
 * compiler specific ugly hack.
 * The way gcc >= 3.4 works is closer to the C++ standard 17.7.3/15.
 */
#if __GNUC__ == 3 && __GNUC_MINOR__ < 4

#define DEFINE_REQUEST_FACTORY_STORAGE( Class ) \
    glite::data::srm::util::details::InstanceStorage<glite::data::srm::util::details::RequestFactory<Class> >\
        glite::data::srm::util::details::RequestFactory<Class>::s_instances;

#define DEFINE_ASYNCH_REQUEST_FACTORY_STORAGE( Class ) \
    glite::data::srm::util::details::InstanceStorage<glite::data::srm::util::details::AsynchRequestFactory<Class> > \
        glite::data::srm::util::details::AsynchRequestFactory<Class>::s_instances;

#else

#define DEFINE_REQUEST_FACTORY_STORAGE( Class ) \
    template <class C> glite::data::srm::util::details::InstanceStorage<glite::data::srm::util::details::RequestFactory<C> > \
        glite::data::srm::util::details::RequestFactory<C>::s_instances; \
    template glite::data::srm::util::details::InstanceStorage<glite::data::srm::util::details::RequestFactory<Class> > \
        glite::data::srm::util::details::RequestFactory<Class>::s_instances;

#define DEFINE_ASYNCH_REQUEST_FACTORY_STORAGE( Class ) \
    template <class C> glite::data::srm::util::details::InstanceStorage<glite::data::srm::util::details::AsynchRequestFactory<C> > \
        glite::data::srm::util::details::AsynchRequestFactory<C>::s_instances; \
    template glite::data::srm::util::details::InstanceStorage<glite::data::srm::util::details::AsynchRequestFactory<Class> >\
        glite::data::srm::util::details::AsynchRequestFactory<Class>::s_instances;

#endif

/*
 * Declare a Factory for the Request Implementation Class
 */
#define DECLARE_REQUEST_FACTORY( Class , RequestClass, Version, ID) \
    private: \
    class ThisRequestFactory ## ID: public glite::data::srm::util::details::RequestFactory<RequestClass> { \
    protected: \
        virtual RequestClass * createRequest(glite::data::srm::util::Context::Ptr ctx) /*throw (LogicError)*/ { \
            return new Class(ctx); \
        } \
        virtual bool isRequestSupported(glite::data::srm::util::Context::Context::Ptr ctx) /*throw ()*/ { \
            return true; \
        } \
    private: \
        friend class Class; \
        ThisRequestFactory ## ID():glite::data::srm::util::details::RequestFactory<RequestClass>(Version){} \
    }; \
    static ThisRequestFactory ## ID s_request_factory ## ID;
    
#define DECLARE_REQUEST_FACTORY_UNSUPPORTED( Class , RequestClass, Version, ID) \
    private: \
    class ThisRequestFactory ## ID: public glite::data::srm::util::details::RequestFactory<RequestClass> { \
    protected: \
        virtual RequestClass * createRequest(glite::data::srm::util::Context::Ptr ctx) /*throw (LogicError)*/ { \
            return new Class(ctx); \
        } \
        virtual bool isRequestSupported(glite::data::srm::util::Context::Context::Ptr ctx) /*throw ()*/ { \
            return false; \
        } \
    private: \
        friend class Class; \
        ThisRequestFactory ## ID():glite::data::srm::util::details::RequestFactory<RequestClass>(Version){} \
    }; \
    static ThisRequestFactory ## ID s_request_factory ## ID;
    
    
#define DECLARE_ASYNCH_REQUEST_FACTORY( Class , AsynchRequestClass, Version, ID) \
    private: \
    class ThisAsynchRequestFactory ## ID: public glite::data::srm::util::details::AsynchRequestFactory<AsynchRequestClass> { \
    protected: \
        virtual AsynchRequestClass * createRequest(glite::data::srm::util::Context::Ptr ctx, const std::string& token) /*throw (LogicError)*/ { \
            return new Class(ctx,token); \
        } \
    private: \
        friend class Class; \
        ThisAsynchRequestFactory ## ID():glite::data::srm::util::details::AsynchRequestFactory<AsynchRequestClass>(Version){} \
    }; \
    static ThisAsynchRequestFactory ## ID s_asynch_request_factory ## ID;
    
/*
 * Define the Factory for the Request Implementation Class
 */
#define DEFINE_REQUEST_FACTORY( Class , ID) \
    Class::ThisRequestFactory ## ID Class::s_request_factory ## ID;

/*
 * Define the Factory for the Asynch Request Implementation Class
 */
#define DEFINE_ASYNCH_REQUEST_FACTORY( Class , ID) \
    Class::ThisAsynchRequestFactory ## ID Class::s_asynch_request_factory ## ID;

} // End namespace details
} // End namespace util
} // End namespace srm
} // End namespace data
} // End namespace glite

#endif //GLITE_DATA_SRM_UTIL_CPP_DETAILS_REQUEST_FACTORY_H_
