/**
 *  Copyright (c) Members of the EGEE Collaboration. 2004. 
 *  See http://www.eu-egee.org/partners/ for details on the copyright
 *  holders.  
 *
 *  Licensed under the Apache License, Version 2.0 (the "License"); 
 *  you may not use this file except in compliance with the License. 
 *  You may obtain a copy of the License at 
 *
 *      http://www.apache.org/licenses/LICENSE-2.0 
 *
 *  Unless required by applicable law or agreed to in writing, software 
 *  distributed under the License is distributed on an "AS IS" BASIS, 
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 *  See the License for the specific language governing permissions and 
 *  limitations under the License.
 *
 *  GLite Data SRM Util CPP: Glite Types
 *
 *  Authors: Paolo Badino <paolo.badino@cern.ch>, Paolo Tedesco <paolo.tedesco@cern.ch>
 *  Version info: $Id: types.h,v 1.11 2008/10/15 07:06:31 ptedesco Exp $
 *  Release: $Name: glite-data-srm-util-cpp_R_2_0_0_1 $
 *
 */
#ifndef GLITE_DATA_SRM_UTIL_GLITE_TYPES_H_
#define GLITE_DATA_SRM_UTIL_GLITE_TYPES_H_

#include <boost/cstdint.hpp>
#include <string>
#include <vector>
#include <map>
#include <ostream>

// TODO This common definitions should go in a org.glite.data.util-cpp module
#include "glite/data/agents/glite-types.h"
#include "glite/data/agents/AgentExceptions.h"

namespace glite {
namespace data  {
namespace srm   {
namespace util  {

/*
 * define basic types
 */
typedef glite::data::agents::longlong       longlong;
typedef std::vector<std::string>            StringArray;

/*
 * Define Types (SRM 2.2 Standard is used as Reference)
 */
typedef std::map<std::string,std::string>   ExtraInfo;

// -----------------------------------------------------------------------------
// Enums
// -----------------------------------------------------------------------------
struct AccessPattern{
    typedef enum {
        UNDEF           = -1,
        TRANSFER_MODE   = 0,
        PROCESSING_MODE,
    } Value;
    
    /**
     * Enum to string convertion function
     */
    static const char * toString(Value value) /*throw()*/;
};

struct ConnectionType{
    typedef enum{
        UNDEF = -1,
        WAN   =  0,
        LAN
    } Value;
    
    /**
     * Enum to string convertion function
     */
    static const char * toString(Value value) /*throw()*/;
};

struct FileStorageType{
    typedef enum {
        DEFAULT = 0,
        VOLATILE,
        DURABLE,
        PERMANENT
    } Value;
    
    /**
     * Enum to string convertion function
     */
    static const char * toString(Value value) /*throw()*/;
};

struct RetentionPolicy{
    typedef enum {
        DEFAULT = 0,
        REPLICA,
        OUTPUT,
        CUSTODIAL
    } Value;
    
    /**
     * Enum to string convertion function
     */
    static const char * toString(Value value) /*throw()*/;
};

struct AccessLatency{
    typedef enum {
        DEFAULT = 0,
        ONLINE,
        NEARLINE
    } Value;
    
    /**
     * Enum to string convertion function
     */
    static const char * toString(Value value) /*throw()*/;
};

struct OverwriteMode {
    typedef enum {
        DEFAULT = 0,
        NEVER,
        ALWAYS,
        WHEN_FILES_ARE_DIFFERENT
    } Value;
    
    /**
     * Enum to string convertion function
     */
    static const char * toString(Value value) /*throw()*/;
};


struct FileLocality {
    typedef enum {
        UNDEF  = -1,
        ONLINE,
        NEARLINE,
        ONLINE_AND_NEARLINE,
        LOST,
        NONE,
        UNAVAILABLE
    } Value;
    
    /**
     * Enum to string convertion function
     */
    static const char * toString(Value value) /*throw()*/;
};

struct FileType {
    typedef enum {
        UNDEF,
        FILE,
        DIRECTORY,
        LINK
    } Value;
    
    /**
     * Enum to string convertion function
     */
    static const char * toString(Value value) /*throw()*/;
};

struct PermissionMode {
    typedef enum {
        NONE       = 00,
        X          = 01,
        W          = 02,
        WX         = 03,
        R          = 04,
        RX         = 05,
        RW         = 06,
        RWX        = 07
    } Value;
    
    /**
     * Enum to string convertion function
     */
    static const char * toString(Value value) /*throw()*/;
};

// -----------------------------------------------------------------------------
// Structs
// -----------------------------------------------------------------------------

struct SrmStatus{
    typedef enum {
    SRM_UNDEF  = -9999,
    SRM_SUCCESS = 0,
    SRM_FAILURE = -1,
    SRM_AUTHENTICATION_FAILURE = -2,
    SRM_AUTHORIZATION_FAILURE = -3,
    SRM_INVALID_REQUEST = -4,
    SRM_INVALID_PATH = -5,
    SRM_FILE_LIFETIME_EXPIRED = -6,
    SRM_SPACE_LIFETIME_EXPIRED = -7,
    SRM_EXCEED_ALLOCATION = -8,
    SRM_NO_USER_SPACE = -9,
    SRM_NO_FREE_SPACE = -10,
    SRM_DUPLICATION_ERROR = -11,
    SRM_NON_EMPTY_DIRECTORY = -12,
    SRM_TOO_MANY_RESULTS = -13,
    SRM_INTERNAL_ERROR = -14,
    SRM_FATAL_INTERNAL_ERROR = -15,
    SRM_NOT_SUPPORTED = -16,
    SRM_REQUEST_QUEUED = 1,
    SRM_REQUEST_INPROGRESS = 2,
    SRM_REQUEST_SUSPENDED = 3,
    SRM_ABORTED = 4,
    SRM_RELEASED = 5,
    SRM_FILE_PINNED = 6,
    SRM_FILE_IN_CACHE = 7,
    SRM_SPACE_AVAILABLE = 8,
    SRM_LOWER_SPACE_GRANTED = 9,
    SRM_DONE = 10,
    SRM_PARTIAL_SUCCESS = 11,
    SRM_REQUEST_TIMED_OUT = 12,
    SRM_LAST_COPY = 13,
    SRM_FILE_BUSY = 14,
    SRM_FILE_LOST = 15,
    SRM_FILE_UNAVAILABLE = 16,
    SRM_CUSTOM_STATUS = 9999     
    } Code;    
    
    Code code;
    std::string   message;
    
    /**
     * Constructor
     */
    SrmStatus():
        code(SRM_UNDEF){}
    /**
     * Constructor
     */
    SrmStatus(Code c,const std::string& m):
        code(c),message(m){}

    /**
     * Enum to string convertion functions
     */
    static const char * toString(Code code) /*throw()*/;
};

/**
 * UserPermission
 */
struct UserPermission {
    std::string           userID;
    PermissionMode::Value mode;
    
    UserPermission():mode(PermissionMode::NONE){}
};

/**
 * GroupPermission
 */
struct GroupPermission {
    std::string           groupID;
    PermissionMode::Value mode;
    
    GroupPermission():mode(PermissionMode::NONE){}
};

/**
 * FileRequest
 */
struct FileRequest{
    std::string   surl;
    SrmStatus     status;
    longlong      size;
    unsigned long extimatedWaitTime;
    unsigned long remainingPinTime;
    
    FileRequest():
        size(0),
        extimatedWaitTime((unsigned long)-1),
        remainingPinTime((unsigned long)-1){}
};

/**
 * GetFileRequest
 */
struct GetFileRequest: public FileRequest{
    std::string   turl;
};

/**
 * PutFileRequest
 */
struct PutFileRequest: public FileRequest{
    std::string   turl;
    unsigned long remainingFileTime;
    
    PutFileRequest():
        FileRequest(),
        remainingFileTime((unsigned long)-1){}
};

/**
 * BringOnlineFileRequest
 */
struct BringOnlineFileRequest: public FileRequest{
};

/**
 * CopyFileRequest
 */
struct CopyFileRequest{
    std::string   sourceSurl;
    std::string   targetSurl;
    SrmStatus     status;
    longlong      size;
    unsigned long extimatedWaitTime;
    unsigned long remainingFileTime;
    
    CopyFileRequest():
        size(0),
        extimatedWaitTime((unsigned long)-1),
        remainingFileTime((unsigned long)-1){}
};

/**
 * RmFileRequest
 */
struct RmFileRequest{
    std::string   surl;
    SrmStatus     status;
    
    RmFileRequest(){}
};

/**
 * AbortFileRequest
 */
typedef RmFileRequest AbortFileRequest;

/**
 * TransferParameters
 */
struct TransferParameters{
    AccessPattern::Value  accessPattern;
    ConnectionType::Value connectionType;
    std::string           protocol;
    
    TransferParameters():
        accessPattern(AccessPattern::TRANSFER_MODE),
        connectionType(ConnectionType::WAN){}
};    
    
/**
 * RetentionPolicyInfo
 */
struct RetentionPolicyInfo{
    RetentionPolicy::Value retentionPolicy;
    AccessLatency::Value   accessLatency;
    
    RetentionPolicyInfo():
        retentionPolicy(RetentionPolicy::DEFAULT),
        accessLatency(AccessLatency::DEFAULT){}
};    

/**
 * MetaDataPathDetail
 */
struct MetaDataPathDetail{
    std::string            surl;
    SrmStatus              status;
    longlong               size;
    time_t                 createdAtTime;
    time_t                 lastModificationTime;
    FileStorageType::Value fileStorageType;
    RetentionPolicyInfo    retentionPolicyInfo;
    FileLocality::Value    fileLocality;
    StringArray            spaceTokens;
    FileType::Value        fileType;
    int                    lifetimeAssigned;
    int                    lifetimeLeft;
    UserPermission         ownerPermission;
    GroupPermission        groupPermission;
    PermissionMode::Value  otherPermission;
    std::string            checkSumType;
    std::string            checkSumValue;
    
    MetaDataPathDetail():
        size(0),
        createdAtTime((time_t)-1),
        lastModificationTime((time_t)-1),
        fileStorageType(FileStorageType::DEFAULT),
        fileLocality(FileLocality::NONE),
        fileType(FileType::UNDEF),
        lifetimeAssigned(-1),
        lifetimeLeft(-1),
        otherPermission(PermissionMode::NONE){}
};

/**
 * MetaDataPathDetail2
 * Adds the arrayOfSubPaths field (ignored by MetaDataPathDetail).
 * Used by SrmLs, differentiated to maintain bacward compatibility.
 */
struct MetaDataPathDetail2 : public MetaDataPathDetail{

    std::vector<MetaDataPathDetail2> arrayOfSubPaths;
    
    MetaDataPathDetail2() : MetaDataPathDetail(){
    }
    
};

/**
 * MetaDataSpace
 */
struct MetaDataSpace{
    std::string         spaceToken;
    SrmStatus           status;
    RetentionPolicyInfo retentionPolicyInfo;
    std::string         owner;
    longlong            totalSize; // best effort
    longlong            guaranteedSize;
    longlong            unusedSize;
    int                 lifetimeAssigned;
    int                 lifetimeLeft;
    
    MetaDataSpace() : 
        totalSize(0),
        guaranteedSize(0),
        unusedSize(0),
        lifetimeAssigned(0),
        lifetimeLeft(0){}
    
    
};

} // End namespace util
} // End namespace srm
} // End namespace data
} // End namespace glite

namespace std{
// overloads of std::ostream operator << 
std::ostream& operator << (std::ostream& stream,glite::data::srm::util::AccessPattern::Value value);
std::ostream& operator << (std::ostream& stream,glite::data::srm::util::ConnectionType::Value value);
std::ostream& operator << (std::ostream& stream,glite::data::srm::util::FileStorageType::Value value);
std::ostream& operator << (std::ostream& stream,glite::data::srm::util::RetentionPolicy::Value value);
std::ostream& operator << (std::ostream& stream,glite::data::srm::util::AccessLatency::Value value);
std::ostream& operator << (std::ostream& stream,glite::data::srm::util::OverwriteMode::Value value);
std::ostream& operator << (std::ostream& stream,glite::data::srm::util::FileLocality::Value value);
std::ostream& operator << (std::ostream& stream,glite::data::srm::util::FileType::Value value);
std::ostream& operator << (std::ostream& stream,glite::data::srm::util::PermissionMode::Value value);
std::ostream& operator << (std::ostream& stream,glite::data::srm::util::SrmStatus::Code value);
std::ostream& operator << (std::ostream& stream,const glite::data::srm::util::SrmStatus& value);
} // namespace std

#endif //GLITE_DATA_SRM_UTIL_GLITE_TYPES_H_

